package org.jboss.security.plugins;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.KeyStore;
import java.util.Arrays;
import java.util.StringTokenizer;
import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.TrustManagerFactory;
import javax.security.auth.callback.CallbackHandler;
import org.jboss.mx.util.MBeanServerLocator;
import org.jboss.security.SecurityDomain;
import org.jboss.security.Util;
import org.jboss.security.auth.callback.SecurityAssociationHandler;
import org.jboss.security.plugins.JaasSecurityDomainMBean;
import org.jboss.security.plugins.JaasSecurityManager;
import org.jboss.security.plugins.JaasSecurityManagerServiceMBean;
import org.jboss.security.plugins.SubjectActions;

public class JaasSecurityDomain extends JaasSecurityManager implements SecurityDomain, JaasSecurityDomainMBean {

   private static final RuntimePermission encodePermission = new RuntimePermission("org.jboss.security.plugins.JaasSecurityDomain.encode");
   private static final RuntimePermission decodePermission = new RuntimePermission("org.jboss.security.plugins.JaasSecurityDomain.decode");
   private KeyStore keyStore;
   private KeyManagerFactory keyMgr;
   private String keyStoreType;
   private URL keyStoreURL;
   private char[] keyStorePassword;
   private String keyStorePasswordCmd;
   private String keyStorePasswordCmdType;
   private SecretKey cipherKey;
   private String cipherAlgorithm;
   private byte[] salt;
   private int iterationCount;
   private PBEParameterSpec cipherSpec;
   private ObjectName managerServiceName;
   private KeyStore trustStore;
   private String trustStoreType;
   private char[] trustStorePassword;
   private URL trustStoreURL;
   private TrustManagerFactory trustMgr;


   public JaasSecurityDomain() {
      this.keyStoreType = "JKS";
      this.cipherAlgorithm = "PBEwithMD5andDES";
      this.salt = new byte[]{(byte)1, (byte)2, (byte)3, (byte)4, (byte)5, (byte)6, (byte)7, (byte)8};
      this.iterationCount = 103;
      this.managerServiceName = JaasSecurityManagerServiceMBean.OBJECT_NAME;
      this.trustStoreType = "JKS";
   }

   public JaasSecurityDomain(String securityDomain) {
      this(securityDomain, new SecurityAssociationHandler());
   }

   public JaasSecurityDomain(String securityDomain, CallbackHandler handler) {
      super(securityDomain, handler);
      this.keyStoreType = "JKS";
      this.cipherAlgorithm = "PBEwithMD5andDES";
      this.salt = new byte[]{(byte)1, (byte)2, (byte)3, (byte)4, (byte)5, (byte)6, (byte)7, (byte)8};
      this.iterationCount = 103;
      this.managerServiceName = JaasSecurityManagerServiceMBean.OBJECT_NAME;
      this.trustStoreType = "JKS";
   }

   public KeyStore getKeyStore() throws SecurityException {
      return this.keyStore;
   }

   public KeyManagerFactory getKeyManagerFactory() throws SecurityException {
      return this.keyMgr;
   }

   public KeyStore getTrustStore() throws SecurityException {
      return this.trustStore;
   }

   public TrustManagerFactory getTrustManagerFactory() throws SecurityException {
      return this.trustMgr;
   }

   public ObjectName getManagerServiceName() {
      return this.managerServiceName;
   }

   public void setManagerServiceName(ObjectName managerServiceName) {
      this.managerServiceName = managerServiceName;
   }

   public String getKeyStoreType() {
      return this.keyStoreType;
   }

   public void setKeyStoreType(String type) {
      this.keyStoreType = type;
   }

   public String getKeyStoreURL() {
      String url = null;
      if(this.keyStoreURL != null) {
         url = this.keyStoreURL.toExternalForm();
      }

      return url;
   }

   public void setKeyStoreURL(String storeURL) throws IOException {
      this.keyStoreURL = this.validateStoreURL(storeURL);
      this.log.debug("Using KeyStore=" + this.keyStoreURL.toExternalForm());
   }

   public void setKeyStorePass(String password) {
      this.keyStorePassword = null;
      if(password.charAt(0) == 123) {
         StringTokenizer tokenizer = new StringTokenizer(password, "{}");
         this.keyStorePasswordCmdType = tokenizer.nextToken();
         this.keyStorePasswordCmd = tokenizer.nextToken();
      } else {
         this.keyStorePassword = password.toCharArray();
      }

   }

   public String getTrustStoreType() {
      return this.trustStoreType;
   }

   public void setTrustStoreType(String type) {
      this.trustStoreType = type;
   }

   public void setTrustStorePass(String password) {
      this.trustStorePassword = password.toCharArray();
   }

   public String getTrustStoreURL() {
      String url = null;
      if(this.trustStoreURL != null) {
         url = this.trustStoreURL.toExternalForm();
      }

      return url;
   }

   public void setTrustStoreURL(String storeURL) throws IOException {
      this.trustStoreURL = this.validateStoreURL(storeURL);
   }

   public void setSalt(String salt) {
      this.salt = salt.getBytes();
   }

   public void setIterationCount(int iterationCount) {
      this.iterationCount = iterationCount;
   }

   public String getCipherAlgorithm() {
      return this.cipherAlgorithm;
   }

   public void setCipherAlgorithm(String cipherAlgorithm) {
      this.cipherAlgorithm = cipherAlgorithm;
   }

   public String getName() {
      return "JaasSecurityDomain(" + this.getSecurityDomain() + ")";
   }

   public byte[] encode(byte[] secret) throws Exception {
      SecurityManager sm = System.getSecurityManager();
      if(sm != null) {
         System.out.println("Checking: " + encodePermission);
         sm.checkPermission(encodePermission);
      }

      Cipher cipher = Cipher.getInstance(this.cipherAlgorithm);
      cipher.init(1, this.cipherKey, this.cipherSpec);
      byte[] encoding = cipher.doFinal(secret);
      return encoding;
   }

   public byte[] decode(byte[] secret) throws Exception {
      SecurityManager sm = System.getSecurityManager();
      if(sm != null) {
         sm.checkPermission(decodePermission);
      }

      Cipher cipher = Cipher.getInstance(this.cipherAlgorithm);
      cipher.init(2, this.cipherKey, this.cipherSpec);
      byte[] decode = cipher.doFinal(secret);
      return decode;
   }

   public String encode64(byte[] secret) throws Exception {
      byte[] encoding = this.encode(secret);
      String b64 = Util.tob64(encoding);
      return b64;
   }

   public byte[] decode64(String secret) throws Exception {
      byte[] encoding = Util.fromb64(secret);
      byte[] decode = this.decode(encoding);
      return decode;
   }

   public void reloadKeyAndTrustStore() throws Exception {
      this.loadKeyAndTrustStore();
   }

   protected void startService() throws Exception {
      this.loadKeystorePassword();
      this.loadKeyAndTrustStore();
      if(this.managerServiceName != null) {
         MBeanServer server = MBeanServerLocator.locateJBoss();
         Object[] params = new Object[]{this.getSecurityDomain(), this};
         String[] signature = new String[]{"java.lang.String", "org.jboss.security.SecurityDomain"};
         server.invoke(this.managerServiceName, "registerSecurityDomain", params, signature);
      }

   }

   protected void stopService() {
      if(this.keyStorePassword != null) {
         Arrays.fill(this.keyStorePassword, '\u0000');
         this.keyStorePassword = null;
      }

      this.cipherKey = null;
   }

   private void loadKeystorePassword() throws Exception {
      if(this.keyStorePassword == null) {
         if(this.keyStorePasswordCmdType.equals("EXT")) {
            this.execPasswordCmd();
         } else {
            if(!this.keyStorePasswordCmdType.equals("CLASS")) {
               throw new IllegalArgumentException("Unknown keyStorePasswordCmdType: " + this.keyStorePasswordCmdType);
            }

            this.invokePasswordClass();
         }
      }

      this.cipherSpec = new PBEParameterSpec(this.salt, this.iterationCount);
      PBEKeySpec keySpec = new PBEKeySpec(this.keyStorePassword);
      SecretKeyFactory factory = SecretKeyFactory.getInstance("PBEwithMD5andDES");
      this.cipherKey = factory.generateSecret(keySpec);
   }

   private void loadKeyAndTrustStore() throws Exception {
      InputStream algorithm;
      String algorithm1;
      if(this.keyStoreURL != null) {
         this.keyStore = KeyStore.getInstance(this.keyStoreType);
         algorithm = this.keyStoreURL.openStream();
         this.keyStore.load(algorithm, this.keyStorePassword);
         algorithm1 = KeyManagerFactory.getDefaultAlgorithm();
         this.keyMgr = KeyManagerFactory.getInstance(algorithm1);
         this.keyMgr.init(this.keyStore, this.keyStorePassword);
      }

      if(this.trustStoreURL != null) {
         this.trustStore = KeyStore.getInstance(this.trustStoreType);
         algorithm = this.trustStoreURL.openStream();
         this.trustStore.load(algorithm, this.trustStorePassword);
         algorithm1 = TrustManagerFactory.getDefaultAlgorithm();
         this.trustMgr = TrustManagerFactory.getInstance(algorithm1);
         this.trustMgr.init(this.trustStore);
      } else if(this.keyStore != null) {
         this.trustStore = this.keyStore;
         String algorithm2 = TrustManagerFactory.getDefaultAlgorithm();
         this.trustMgr = TrustManagerFactory.getInstance(algorithm2);
         this.trustMgr.init(this.trustStore);
      }

   }

   private void execPasswordCmd() throws Exception {
      this.log.debug("Executing command: " + this.keyStorePasswordCmd);
      Runtime rt = Runtime.getRuntime();
      Process p = rt.exec(this.keyStorePasswordCmd);
      InputStream stdin = p.getInputStream();
      BufferedReader reader = new BufferedReader(new InputStreamReader(stdin));
      String password = reader.readLine();
      stdin.close();
      int exitCode = p.waitFor();
      this.log.debug("Command exited with: " + exitCode);
      this.keyStorePassword = password.toCharArray();
   }

   private void invokePasswordClass() throws Exception {
      this.keyStorePassword = null;
      String classname = this.keyStorePasswordCmd;
      String ctorArg = null;
      int colon = this.keyStorePasswordCmd.indexOf(58);
      if(colon > 0) {
         classname = this.keyStorePasswordCmd.substring(0, colon);
         ctorArg = this.keyStorePasswordCmd.substring(colon + 1);
      }

      this.log.debug("Loading class: " + classname + ", ctorArg=" + ctorArg);
      ClassLoader loader = SubjectActions.getContextClassLoader();
      Class c = loader.loadClass(classname);
      Object instance = null;
      Class[] e;
      Object[] args;
      if(ctorArg != null) {
         e = new Class[]{String.class};
         Constructor tmp = c.getConstructor(e);
         args = new Object[]{ctorArg};
         instance = tmp.newInstance(args);
      } else {
         instance = c.newInstance();
      }

      try {
         this.log.debug("Checking for toCharArray");
         e = new Class[0];
         Method tmp2 = c.getMethod("toCharArray", e);
         args = new Object[0];
         this.log.debug("Invoking toCharArray");
         this.keyStorePassword = (char[])tmp2.invoke(instance, args);
      } catch (NoSuchMethodException var10) {
         this.log.debug("No toCharArray found, invoking toString");
         String tmp1 = instance.toString();
         if(tmp1 != null) {
            this.keyStorePassword = tmp1.toCharArray();
         }
      }

   }

   private URL validateStoreURL(String storeURL) throws IOException {
      URL url = null;

      try {
         url = new URL(storeURL);
      } catch (MalformedURLException var4) {
         ;
      }

      if(url == null) {
         File msg = new File(storeURL);
         if(msg.exists()) {
            url = msg.toURL();
         }
      }

      if(url == null) {
         ClassLoader msg1 = SubjectActions.getContextClassLoader();
         url = msg1.getResource(storeURL);
      }

      if(url == null) {
         String msg2 = "Failed to find url=" + storeURL + " as a URL, file or resource";
         throw new MalformedURLException(msg2);
      } else {
         return url;
      }
   }

}
